/* Copyright 2010 AlarmPoint Systems */ 
importPackage(java.lang);
importPackage(java.sql);
importPackage(java.util);
importPackage(java.io);
importPackage(java.net);
importPackage(Packages.javax.xml);
importPackage(Packages.javax.xml.parsers);
importPackage(Packages.javax.xml.transform);
importPackage(Packages.javax.xml.transform.dom);
importPackage(Packages.javax.xml.transform.stream);
importPackage(Packages.org.w3c.dom);
importPackage(Packages.com.alarmpoint.servicemanager);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLMessage);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLToken);
importClass(Packages.com.thoughtworks.xstream.XStream);
importClass(Packages.com.thoughtworks.xstream.converters.reflection.PureJavaReflectionProvider);

var xStream = new XStream(new PureJavaReflectionProvider())

var smUrl = "http://localhost:13080/sc62server/ws";
var exceptionString = "JavaException: com.alarmpoint.servicemanager.ServiceManagerException: ";
var notePrefix = ""; // The Prefix which is added to the note comments for AlarmPoint e.g. "[AlarmPoint] " 
var closureCode = "Fault"; // This should be modified to the appropriate closure code for the integration

// only used for Callout Annotations from the CALLOUT APS script
var calloutAnnotateUser = "falcon";
var calloutAnnotatePass = "";

var smConnection = null;

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by APClient.  The messages may be requests to perform
 * local activity, or they may be requests to make submissions to AlarmPoint.
 * <p>
 * Any APXMLMessage object that this method returns will be sent to AlarmPoint
 * via the Integration Service's outbound queues.
 */
function apia_input(apxml)
{
  return apxml;
}

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by AlarmPoint.  The messages may be responses to
 * previous submissions, or they may be requests to the Integration Service
 * that originate from AlarmPoint
 * (e.g., via an ExternalServiceMessage/ExternalServiceRequest2).
 * <p>
 * Any APXMLMessage object that this method returns will be sent to AlarmPoint
 * either via the Integration Service's outbound queues or directly as a
 * Web Service response, depending on the mechanism used to submit the APXML.
 */
function apia_response(apxml)
{
  var method = apxml.getMethod();
  if (APXMLMessage.OK_METHOD.equalsIgnoreCase(method))
  {
    return handleOK(apxml);
  }
  else if (APXMLMessage.ERROR_METHOD.equalsIgnoreCase(method))
  {
    return handleERROR(apxml);
  }
  else if (APXMLMessage.SEND_METHOD.equalsIgnoreCase(method))
  {
    return handleSend(apxml);
  }
  else if (APXMLMessage.REQUEST_METHOD.equalsIgnoreCase(method))
  {
    return handleRequest(apxml);
  }
  else
  {
    throw new IllegalArgumentException("Unrecognized APXML method: " + method);
  }
}


/**
 * OK responses are returned by AlarmPoint for each APXML message that it accepts
 * from an Integration Service.
 * <p>
 * This implementation just logs the ack.
 */
function handleOK(apxml)
{
  var id = apxml.getTransactionId();
  logInfo("Submission of transaction [" + id + "] succeeded.");
}

/**
 * ERROR responses are returned by AlarmPoint for each APXML message that it
 * is unable to accept from an Integration Service.
 * <p>
 * This implementation just logs the error.
 */
function handleERROR(apxml)
{
  var transaction = apxml.getTransactionId();
  var code = apxml.getValue(APXMLMessage.ERROR_CODE);
  ServiceAPI.getLogger().error("Submission of transaction " + transaction + "failed with error code " + code);
}

/**
 * This implementation just logs que request, there shouldn't be any direct calls to this method
 */
function handleRequest(apxml)
{
  ServiceAPI.getLogger().error("Service received unexpected 'Request' with message: " + apxml);
}



/**
 * The main routine for handling Requests AlarmPoint
 * <p>
 * Throw an exception that will be logged by the Integration Agent.
 */
function handleSend(apxml)
{
  logDebug( "handleSend received apxml: " + apxml ); 

  // handle annotations coming from callout APS script 
  // they are formatted differently 
  // the presence of the message token means this message 
  // is a callout annotation. 
  var message = apxml.getValue("message");
  if ( message != null )
  {
    var incidentId = apxml.getValue("incident_id");
    return handleCalloutAnnotate( incidentId, message );
  }
  else
  {
    return handleApsResponses( apxml );
  }
}
  
  
/**
 * The main routine for handling Requests from AlarmPoint HP SM IM APS
 */  
function handleApsResponses( apxml )
{  
  // message fields
  var incidentId = apxml.getValue("incident_id");
  var requestText = apxml.getValue("request_text");
  var messageText = apxml.getValue("message_text");
 
  var smUser = null;
  var smPass = null;
  var smStatus = null;

  // deserialize the additional tokens
  if (apxml.getValue("additionalTokens") != null && apxml.getValue("additionalTokens") != "") 
  {
    var additionalTokens = xStream.fromXML(apxml.getValue("additionalTokens")); // LinkedHashMap
    
    logDebug( "additionalTokens keys: " + additionalTokens);
    smUser = additionalTokens.get("sm_user"); 
    smPass = additionalTokens.get("sm_pass");
    smStatus = additionalTokens.get("sm_status"); 
  }
  else
  {
    logError("additionalTokens is missing from apxml");
    return; 
  }
  
  if ( smUser == null || smPass == null ) 
  {
    logError("additionalTokens in apxml is missing sm_user or sm_pass which are both manditory ");
    return; 
  }

  logDebug("smUser: " + smUser + " smPass: " + smPass);
  smConnection = ServiceManagerScriptObject( smUrl, smUser, smPass );
  
  logDebug("incident_id: " + incidentId + " requestText: " + requestText);
  logDebug("smStatus: " + smStatus + " messageText: " + messageText);
  if (!incidentId)
  {
     logError("Request Script Failed: Request not associated with an Incident");
     return;  
  }
    
  if (messageText != null && messageText != "")
  {
    messageText = notePrefix + messageText;
  } 
  else 
  {
    messageText = notePrefix + "Updated Incident";
  }

  logDebug("About to call Service Manager requestText: [" + requestText + "]");

  try 
  {
    if (requestText == "annotate" || requestText == "ignore") 
    {
      smConnection.updateIncident(incidentId, messageText);
    }
    else if (requestText == "resolve" )
    {
      // messageText in resolveIncident is ignored so add manual update with this line
      smConnection.updateIncident(incidentId, messageText);
      smConnection.resolveIncident(incidentId, "Resolved by AlarmPoint", "temporary", messageText, closureCode);
    }
    else if (requestText == "reject")
    {
      smConnection.updateIncident(incidentId, messageText, smStatus );  
    }
    else if (requestText == "own") 
    {
       smConnection.updateIncident(incidentId, messageText, smStatus, smUser);
    }
    else
    { 
      logError("Request Script Failed: RequestText: [" + requestText + "] not implemented"); 
      return;
    }
  }
  catch (e)
  {
    logError("Caught exception processing [" + requestText + "] Exception:" + e);
  }
  
  logDebug("Finished call to Service Manager for requestText: [" + requestText + "]");
}

/**
 * handles annotations sent from the Callout APS script
 */
function handleCalloutAnnotate( incidentId, message )
{
  smConnection = ServiceManagerScriptObject( smUrl, calloutAnnotateUser, calloutAnnotatePass );

  logDebug("About to call to Service Manager for callout annotate");

  try 
  {
    smConnection.updateIncident(incidentId, message);
  }
  catch (e)
  {
    logError("Caught exception processing annotation for in handleCalloutAnnotate Exception:" + e);
  }
  
  logDebug("Finished call to Service Manager for callout annotate");
}


/**
 * Retrieves the first incident that matches the specified parameters.
 *
 * Parameters: a java.util.Map<String, String> of query fields and associated values
 * Return: A com.alarmpoint.servicemanager.ServiceManagerIncident object for the first matched incident or a String if a error occurred
 */
function getIncident(params)
{
  if(params == null)
    return "No search parameters specified";
  
  //This search should only ever return 1 result
  var incidents = getIncidents(params);
  
  if(typeof incidents == "string")
    return incidents;
    
  if(incidents.isEmpty())
    return "No Incidents Returned";
    
  return incidents.get(0);
}

/**
 * Retrieves a list of incidents that match the specified parameters
 *
 * Parameters: a java.util.Map<String, String> containing a queryObject field and associated ServiceManagerQuery object or 
 *             a queryString field and associated SM query string or 
 *             query fields and associated values 
 * Return: A java.util.Collection<ServiceManagerIncident> object containing all matching incidents or a String if a error occurred
 */
function getIncidents(params) 
{  
  var sm_user = params.remove("smUser");
  var sm_password = params.remove("smPass");
  var client = new ServiceManagerScriptObject(smUrl, sm_user, sm_password);
  if(params == null)
    return "No search parameters specified";
  var query = "";
  if(params.containsKey("queryObject")){
    query = params.get("queryObject");
  }else if(params.containsKey("queryString")){ 
    query = params.get("queryString");
  }else{ 
    query = new ServiceManagerQuery();
    query.constructFromMap(params);
  }
  try{
    return client.queryIncident(query);
  }catch(e){
    return e.toString().replace(exceptionString, "");
  }
}

/**
 * Saves the changes made to a Service Manager Incident
 *
 * Parameters: a java.util.Map<String, Object> containing a incident field and associated ServiceManagerIncident object and a incidentID field containing the id of the incident to update
 * Returns: null if the update was successful or a String if the update failed
 */
function saveIncident(params) 
{  
  var sm_user = params.remove("smUser");
  var sm_password = params.remove("smPass");
  var client = new ServiceManagerScriptObject(smUrl, sm_user, sm_password);
  var incident = params.get("incident");
  var incidentID = params.get("incidentID");
  
  try{
    var update = incident.constructUpdateInstance();
    if(update.get("IMTicketStatus") != "Closed")
      client.updateIncident(incidentID, incident.constructUpdateInstance());
    else
      client.closeIncident(incidentID, incident.constructUpdateInstance());
    return null;
  }catch(e){
    return e.toString().replace(exceptionString, "");
  }
}

/**
 * Created a new Service Manager Incident
 *
 * Parameters: a java.util.Map<String, Object> containing a incident field and associated ServiceManagerIncident object
 * Returns: A String[] array with the first entry being the id of the new incident created (null if the create failed) and the second entry being the error if the create failed (null if the create succeeded)
 */
function createIncident(params) 
{  
  var sm_user = params.remove("smUser");
  var sm_password = params.remove("smPass");
  var client = new ServiceManagerScriptObject(smUrl, sm_user, sm_password);
  var incident = params.get("incident");
  
  var status = java.lang.reflect.Array.newInstance(java.lang.String, 2);
  try{
    status[0] = client.createIncident(incident.constructUpdateInstance());
    status[1] = null;
    return status;
  }catch(e){
    status[0] = null;
    status[1] = e.toString().replace(exceptionString, "");
    return status;
  }
}

/**
 * Retrieves list values for Category, Subcategory, ProductType and ProblemType predicates.  These lists can either be dynamic lists or static lists
 *
 * Parameters: a java.util.Map<String, boolean>
 * Returns: a java.util.Map<String, Object> containing dynamic lists if the useDynamicLists field is true and static lists otherwise
 */
function getLists(params)
{
  var sm_user = params.remove("smUser");
  var sm_password = params.remove("smPass");
  var client = new ServiceManagerScriptObject(smUrl, sm_user, sm_password);

  try{
    var fields = new HashMap();
    
    fields.put("ClosureCode", client.getSortedList("RetrieveProbcauseListRequest", "cause.code <> \"\"", "cause.code"));
    
    return fields;
  }catch(e){
    return e.toString().replace(exceptionString, "");
  }
}

/**
 * Retrieves a list of objects specified by the Soap operation name and the query
 *
 * Parameters: a java.util.Map containing a operation field and associated SM Soap operation and a query field and associated SM query
 * Returnes: a java.util.Collection<Map<String, String>> containing a list of all the objects that matched the query for the operation and for each object a map of field names to values
 */
function getValueMap(params)
{
  var sm_user = params.remove("smUser");
  var sm_password = params.remove("smPass");
  var client = new ServiceManagerScriptObject(smUrl, sm_user, sm_password);
  var operation = params.get("operation");
  var query = params.get("query");
  
  try{
    return client.getValueMap(operation, query); 
  }catch(e){
    return e.toString().replace(exceptionString, "");
  }
}

/**
 * Used to validate a user SM credentials
 *
 * Returns: null if the validation was successful and a String with the error message otherwise
 */
function validateCreds(params)
{
  var sm_user = params.remove("smUser");
  var sm_password = params.remove("smPass");
  var client = new ServiceManagerScriptObject(smUrl, sm_user, sm_password);
  try{
    client.validateCredentials("RetrieveCategoryListRequest");
  }catch(e){
    return e.toString().replace(exceptionString, "");
  }
  return null; 
}


var PREAMBLE = "Agent Client: HP SM IM - "
function logDebug(msg)
{
    ServiceAPI.getLogger().debug(PREAMBLE + msg);
} 

function logInfo(msg)
{
    ServiceAPI.getLogger().info(PREAMBLE + msg);
} 

function logError(msg)
{
    ServiceAPI.getLogger().error(PREAMBLE + msg);
} 
